from .PRSAW import *

__title__ = "prsaw"
__summary__ = (
    "PRSAW, an acronym for `Python Random Stuff API Wrapper`, is a wrapper for the Random Stuff API."
)
__uri__ = "https://github.com/CodeWithSwastik/prsaw"
__version__ = "0.2.0"
__author__ = "Swas.py"
__email__ = "cwswas.py@gmail.com"
__license__ = "MIT License"
__copyright__ = "Copyright 2021 {}".format(__author__)

def get_version():
    return __version__



import pytest
from prsaw import RandomStuff

from . import joke_response_keys


class TestBlocking:
    def setup(self):
        self.rs = RandomStuff()

    def test_invalid_joke_error(self):
        with pytest.raises(RuntimeError) as exc:
            self.rs.get_joke("deva")
        assert "deva" in str(exc)

    def test_invalid_image_error(self):
        with pytest.raises(RuntimeError) as exc:
            self.rs.get_image("awww")
        assert "awww" in str(exc)

    def test_joke_response(self):
        response = self.rs.get_joke("dev")

        assert isinstance(response, dict)
        assert set(response.keys()).issuperset(joke_response_keys)

    def test_ai_response(self):
        response = self.rs.get_ai_response("Hello!")

        assert isinstance(response, str)

    def test_image_reponse(self):
        response = self.rs.get_image("aww")

        assert isinstance(response, str)

    def teardown(self):
        self.rs.close()



import httpx
from apiclient import APIClient, APIRouter, Get, endpoint


class RandomStuff(APIClient):
    """
    A Wrapper for the Random Stuff API.

    Example Usage:

        rs = RandomStuff()
        joke = rs.get_joke()
        print(joke)
        rs.close()

    Example async usage:
        rs = RandomStuff(async_mode=True)
        joke = await rs.get_joke()
        print(joke)
        await rs.close()
    """
    base_url = "https://api.pgamerx.com"

    _joke_types = ("dev", "spooky", "pun", "any")
    _image_types = ("aww", "duck", "dog", "cat", "memes",
                    "dankmemes", "holup", "art", "harrypottermemes", "facepalm", "any")

    def __init__(self, *, async_mode=False):
        session = httpx.AsyncClient() if async_mode else httpx.Client()
        super().__init__(session=session)

    def _pre_init(self):
        # Convert to json before returning
        self._post_processors.append(lambda res: res.json())

    @endpoint
    def get_joke(self, _type: str = "any") -> dict:
        _type = _type.lower()
        if _type.lower() not in self._joke_types:
            raise RuntimeError("Unknown joke type provided: {}".format(_type))

        return Get("/joke/" + _type)

    @endpoint
    def get_image(self, _type: str = "any") -> str:
        _type = _type.lower()
        if _type not in self._image_types:
            raise RuntimeError("Unknown image type provided: {}".format(_type))

        return Get("/image/" + _type)

    @endpoint
    def get_ai_response(self, msg: str, *, lang="en") -> str:
        return Get("/ai/response", params={"message": msg, "language": lang})

    def _post_get_image(self, res):
        return res[0]
    _post_get_ai_response = _post_get_image

    def close(self):
        return self.session.close()



from setuptools import setup, find_packages

setup(
  name='prsaw',
  version='0.2.0',
  description='PRSAW, an acronym for `Python Random Stuff API Wrapper`, is a wrapper for the Random Stuff API.',
  long_description=open('README.md').read(),
  long_description_content_type = "text/markdown",
  url = "https://github.com/CodeWithSwastik/prsaw",  
  author='Swas.py',
  author_email='cwswas.py@gmail.com',
  license='MIT', 
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
  ],
  keywords='random stuff, api, wrapper', 
  packages=find_packages(),
  install_requires=['instant-api-client==0.1.1'] 
)



from asyncio import get_event_loop

from prsaw import RandomStuff

from . import joke_response_keys


class TestAsync:
    def setup(self):
        self.rs = RandomStuff(async_mode=True)
        self.loop = get_event_loop()
        self.run = self.loop.run_until_complete

    def test_joke_response(self):
        response = self.run(self.rs.get_joke("dev"))

        assert isinstance(response, dict)
        assert set(response.keys()).issuperset(joke_response_keys)

    def test_image_response(self):
        response = self.run(self.rs.get_image("aww"))

        assert isinstance(response, str)

    def test_ai_response(self):
        response = self.run(self.rs.get_ai_response("Hello!"))

        assert isinstance(response, str)
